// ---------------------------
// class GeneralQuarticSurface
// ---------------------------

// -----------
// constructor
// -----------
GeneralQuarticSurface::GeneralQuarticSurface(void)
{
  for (int i = 0; i < 35; ++i) {
    fA[i] = 0.0;
  }
}

// ----------
// destructor
// ----------
GeneralQuarticSurface::~GeneralQuarticSurface(void)
{
}

// -------
// methods
// -------
void GeneralQuarticSurface::drawOpenGL(void) const
{
}

// --------------------------------
// quartic surface specific methods
// --------------------------------
void GeneralQuarticSurface::setCoefficients(const Ray& genericRay)
{
  fCoefficients[0] = 0.0; // constant-term
  fCoefficients[1] = 0.0; // t-factor
  fCoefficients[2] = 0.0; // t^2-factor
  fCoefficients[3] = 0.0; // t^3-factor
  fCoefficients[4] = 0.0; // t^4-factor
}

Vector3D GeneralQuarticSurface::calcNormal(const Point3D& p) const
{
  double partialDerivativeToX = (4.0 * Math::cube(p.fX) * fA[0]) +
                                (3.0 * Math::sqr(p.fX) * p.fY * fA[1]) +
                                (3.0 * Math::sqr(p.fX) * p.fZ * fA[2]) +
                                (3.0 * Math::sqr(p.fX) * fA[3]) +
                                (2.0 * p.fX * Math::sqr(p.fY) * fA[4]) +
                                (2.0 * p.fX * p.fY * p.fZ * fA[5]) +
                                (2.0 * p.fX * p.fY * fA[6]) +
                                (2.0 * p.fX * Math::sqr(p.fZ) * fA[7]) +
                                (2.0 * p.fX * p.fZ * fA[8]) +
                                (2.0 * p.fX * fA[9]) +
                                (Math::cube(p.fY) * fA[10]) +
                                (Math::sqr(p.fY) * p.fZ * fA[11]) +
                                (Math::sqr(p.fY) * fA[12]) +
                                (p.fY * Math::sqr(p.fZ) * fA[13]) +
                                (p.fY * p.fZ * fA[14]) +
                                (p.fY * fA[15]) +
                                (Math::cube(p.fZ) * fA[16]) +
                                (Math::sqr(p.fZ) * fA[17]) +
                                (p.fZ * fA[18]) +
                                fA[19];

  double partialDerivativeToY = (Math::cube(p.fX) * fA[1]) +
                                (2.0 * Math::sqr(p.fX) * p.fY * fA[4]) +
                                (Math::sqr(p.fX) * p.fZ * fA[5]) +
                                (Math::sqr(p.fX) * fA[6]) +
                                (3.0 * p.fX * Math::sqr(p.fY) * fA[10]) +
                                (2.0 * p.fX * p.fY * p.fZ * fA[11]) +
                                (2.0 * p.fX * p.fY * fA[12]) +
                                (p.fX * Math::cube(p.fZ) * fA[13]) +
                                (p.fX * p.fZ * fA[14]) +
                                (p.fX * fA[15]) +
                                (4.0 * Math::cube(p.fY) * fA[20]) +
                                (3.0 * Math::sqr(p.fY) * p.fZ * fA[21]) +
                                (3.0 * Math::sqr(p.fY) * fA[22]) +
                                (2.0 * p.fY * Math::sqr(p.fZ) * fA[23]) +
                                (2.0 * p.fY * p.fZ * fA[24]) +
                                (2.0 * p.fY * fA[25]) +
                                (Math::cube(p.fZ) * fA[26]) +
                                (Math::sqr(p.fZ) * fA[27]) +
                                (p.fZ * fA[28]) +
                                fA[29];

  double partialDerivativeToZ = 1.0;

  return Vector3D(partialDerivativeToX,partialDerivativeToY,partialDerivativeToZ);
}

// ----------------------------
// class PinchedSurfaceWithHorn
// ----------------------------

// -----------
// constructor
// -----------
PinchedSurfaceWithHorn::PinchedSurfaceWithHorn(void) : CubicSurface()
{
}

// ----------
// destructor
// ----------
PinchedSurfaceWithHorn::~PinchedSurfaceWithHorn(void)
{
}

// -------
// methods
// -------
void PinchedSurfaceWithHorn::drawOpenGL(void) const
{
}

// ------------------------------
// cubic surface specific methods
// ------------------------------
void PinchedSurfaceWithHorn::setCoefficients(const Ray& genericRay)
{
  fCoefficients[0] = -(Math::sqr(genericRay.fStart.fX) *
                       genericRay.fStart.fX) +
                      (2.0 * (genericRay.fStart.fX *
                              genericRay.fStart.fZ)) -
                      (genericRay.fStart.fY *
                       Math::sqr(genericRay.fStart.fZ));

  fCoefficients[1] = -(3.0 * Math::sqr(genericRay.fStart.fX) *
                             genericRay.fDirection.fX) +
                      (2.0 * (genericRay.fStart.fX *
                              genericRay.fDirection.fZ) +
                             (genericRay.fDirection.fX *
                              genericRay.fStart.fZ) -
                             (genericRay.fStart.fY *
                              genericRay.fStart.fZ *
                              genericRay.fDirection.fZ)) -
                      (genericRay.fDirection.fY *
                       Math::sqr(genericRay.fStart.fZ));

  fCoefficients[2] = -(3.0 * genericRay.fStart.fX *
                             Math::sqr(genericRay.fDirection.fX)) +
                      (2.0 * (genericRay.fDirection.fX *
                              genericRay.fDirection.fZ) -
                             (genericRay.fDirection.fY *
                              genericRay.fStart.fZ *
                              genericRay.fDirection.fZ)) -
                      (genericRay.fStart.fY *
                       Math::sqr(genericRay.fDirection.fZ));

  fCoefficients[3] = -(Math::sqr(genericRay.fDirection.fX) *
                       genericRay.fDirection.fX) -
                      (genericRay.fDirection.fY *
                       Math::sqr(genericRay.fDirection.fZ));

  fCoefficients[4] = 0.0;
}

Vector3D PinchedSurfaceWithHorn::calcNormal(const Point3D& p) const
{
  double partialDerivativeToX = -(3.0 * Math::sqr(p.fX)) + (2.0 * p.fZ);
  double partialDerivativeToY = -Math::sqr(p.fZ);
  double partialDerivativeToZ = 2.0 * (p.fX - (p.fY * p.fZ));

  return Vector3D(partialDerivativeToX,partialDerivativeToY,partialDerivativeToZ);
}

// -------------------
// class BladedSurface
// -------------------

// -----------
// constructor
// -----------
BladedSurface::BladedSurface(void) : CubicSurface()
{
}

// ----------
// destructor
// ----------
BladedSurface::~BladedSurface(void)
{
}

// -------
// methods
// -------
void BladedSurface::drawOpenGL(void) const
{
}

// ------------------------------
// cubic surface specific methods
// ------------------------------
void BladedSurface::setCoefficients(const Ray& genericRay)
{
  fCoefficients[0] = Math::sqr(genericRay.fStart.fX) +
                     Math::sqr(genericRay.fStart.fY) +
                     Math::sqr(genericRay.fStart.fZ) +
                     (2.0 * genericRay.fStart.fX *
                            genericRay.fStart.fY *
                            genericRay.fStart.fZ) -
                     1.0;

  fCoefficients[1] = 2.0 * ((genericRay.fStart.fX *
                             genericRay.fDirection.fX) +
                            (genericRay.fStart.fY *
                             genericRay.fDirection.fY) +
                            (genericRay.fStart.fZ *
                             genericRay.fDirection.fZ) +
                            (genericRay.fStart.fX *
                             genericRay.fStart.fY *
                             genericRay.fDirection.fZ) +
                            (genericRay.fStart.fX *
                             genericRay.fDirection.fY *
                             genericRay.fStart.fZ) +
                            (genericRay.fDirection.fX *
                             genericRay.fStart.fY *
                             genericRay.fStart.fZ));

  fCoefficients[2] = Math::sqr(genericRay.fDirection.fX) +
                     Math::sqr(genericRay.fDirection.fY) +
                     Math::sqr(genericRay.fDirection.fZ) +
                     (2.0 * (genericRay.fStart.fX *
                             genericRay.fDirection.fY *
                             genericRay.fDirection.fZ) +
                            (genericRay.fDirection.fX *
                             genericRay.fStart.fY *
                             genericRay.fDirection.fZ) +
                            (genericRay.fDirection.fX *
                             genericRay.fDirection.fY *
                             genericRay.fStart.fZ));

  fCoefficients[3] = 2.0 * (genericRay.fDirection.fX *
                            genericRay.fDirection.fY *
                            genericRay.fDirection.fZ);

  fCoefficients[4] = 0.0;
}

Vector3D BladedSurface::calcNormal(const Point3D& p) const
{
  double partialDerivativeToX = 2.0 * (p.fX + (p.fY * p.fZ));
  double partialDerivativeToY = 2.0 * (p.fY + (p.fX * p.fZ));
  double partialDerivativeToZ = 2.0 * (p.fZ + (p.fX * p.fY));

  return Vector3D(partialDerivativeToX,partialDerivativeToY,partialDerivativeToZ);
}

// ------------------
// class StrangeCubic
// ------------------

// -----------
// constructor
// -----------
StrangeCubic::StrangeCubic(void) : CubicSurface()
{
}

// ----------
// destructor
// ----------
StrangeCubic::~StrangeCubic(void)
{
}

// -------
// methods
// -------
void StrangeCubic::drawOpenGL(void) const
{
}

// ------------------------------
// cubic surface specific methods
// ------------------------------
void StrangeCubic::setCoefficients(const Ray& genericRay)
{
  double a = 0.11111;

  fCoefficients[0] = (Math::sqr(genericRay.fStart.fX) *
                      genericRay.fStart.fX) +
                     Math::sqr(genericRay.fStart.fY) -
                     (a * genericRay.fStart.fX *
                          Math::sqr(genericRay.fStart.fZ));

  fCoefficients[1] = (3.0 * Math::sqr(genericRay.fStart.fX) *
                            genericRay.fDirection.fX) +
                     (2.0 * ((genericRay.fStart.fY *
                              genericRay.fDirection.fY) -
                             (a * genericRay.fStart.fX *
                                  genericRay.fStart.fZ *
                                  genericRay.fDirection.fZ))) -
                     (a * genericRay.fDirection.fX *
                          Math::sqr(genericRay.fStart.fZ));

  fCoefficients[2] = (3.0 * genericRay.fStart.fX *
                            Math::sqr(genericRay.fDirection.fX)) +
                     Math::sqr(genericRay.fDirection.fY) -
                     (a * (2.0 * genericRay.fDirection.fX *
                                 genericRay.fStart.fZ *
                                 genericRay.fDirection.fZ) +
                          (genericRay.fStart.fX *
                           Math::sqr(genericRay.fDirection.fZ)));

  fCoefficients[3] = (Math::sqr(genericRay.fDirection.fX) *
                      genericRay.fDirection.fX) -
                     (a * genericRay.fDirection.fX *
                          Math::sqr(genericRay.fDirection.fZ));

  fCoefficients[4] = 0.0;
}

Vector3D StrangeCubic::calcNormal(const Point3D& p) const
{
  double a = 0.11111;

  double partialDerivativeToX = (3.0 * Math::sqr(p.fX)) - (a * Math::sqr(p.fZ));
  double partialDerivativeToY = 2.0 * p.fY;
  double partialDerivativeToZ = -(2.0 * a * p.fX * p.fZ);

  return Vector3D(partialDerivativeToX,partialDerivativeToY,partialDerivativeToZ);
}

// ------------------
// class KleinQuartic
// ------------------

// -----------
// constructor
// -----------
KleinQuartic::KleinQuartic(void) : QuarticSurface()
{
}

// ----------
// destructor
// ----------
KleinQuartic::~KleinQuartic(void)
{
}

// -------
// methods
// -------
void KleinQuartic::drawOpenGL(void) const
{
}

// --------------------------------
// quartic surface specific methods
// --------------------------------
void KleinQuartic::setCoefficients(const Ray& genericRay)
{
  fCoefficients[0] = (Math::cube(genericRay.fStart.fX) *
                      genericRay.fStart.fY) +
                     (Math::cube(genericRay.fStart.fY) *
                      genericRay.fStart.fZ) +
                     (Math::cube(genericRay.fStart.fZ) *
                      genericRay.fStart.fX);

  fCoefficients[1] = (Math::cube(genericRay.fStart.fX) *
                      genericRay.fDirection.fY) +
                     (Math::cube(genericRay.fStart.fY) *
                      genericRay.fDirection.fZ) +
                     (Math::cube(genericRay.fStart.fZ) *
                      genericRay.fDirection.fX) +
                     3.0 * ((Math::sqr(genericRay.fStart.fX) *
                             genericRay.fDirection.fX *
                             genericRay.fStart.fY) +
                            (Math::sqr(genericRay.fStart.fY) *
                             genericRay.fDirection.fY *
                             genericRay.fStart.fZ) +
                            (Math::sqr(genericRay.fStart.fZ) *
                             genericRay.fDirection.fZ *
                             genericRay.fStart.fX));

  fCoefficients[2] = 3.0 * ((Math::sqr(genericRay.fStart.fX) *
                             genericRay.fDirection.fX *
                             genericRay.fDirection.fY) +
                            (Math::sqr(genericRay.fStart.fY) *
                             genericRay.fDirection.fY *
                             genericRay.fDirection.fZ) +
                            (Math::sqr(genericRay.fStart.fZ) *
                             genericRay.fDirection.fZ *
                             genericRay.fDirection.fX) +
                            (genericRay.fStart.fX *
                             Math::sqr(genericRay.fDirection.fX) *
                             genericRay.fStart.fY) +
                            (genericRay.fStart.fY *
                             Math::sqr(genericRay.fDirection.fY) *
                             genericRay.fStart.fZ) +
                            (genericRay.fStart.fZ *
                             Math::sqr(genericRay.fDirection.fZ) *
                             genericRay.fStart.fX));

  fCoefficients[3] = (Math::cube(genericRay.fDirection.fX) *
                      genericRay.fStart.fY) +
                     (Math::cube(genericRay.fDirection.fY) *
                      genericRay.fStart.fZ) +
                     (Math::cube(genericRay.fDirection.fZ) *
                      genericRay.fStart.fX) +
                     (3.0 * ((genericRay.fStart.fX *
                              Math::sqr(genericRay.fDirection.fX) *
                              genericRay.fDirection.fY) +
                             (genericRay.fStart.fY *
                              Math::sqr(genericRay.fDirection.fY) *
                              genericRay.fDirection.fZ) +
                             (genericRay.fStart.fZ *
                              Math::sqr(genericRay.fDirection.fZ) *
                              genericRay.fDirection.fX)));

  fCoefficients[4] = (Math::cube(genericRay.fDirection.fX) *
                      genericRay.fDirection.fY) +
                     (Math::cube(genericRay.fDirection.fY) *
                      genericRay.fDirection.fZ) +
                     (Math::cube(genericRay.fDirection.fZ) *
                      genericRay.fDirection.fX);
}

Vector3D KleinQuartic::calcNormal(const Point3D& p) const
{
  double partialDerivativeToX = (3.0 * Math::sqr(p.fX) * p.fY) + (Math::sqr(p.fZ) * p.fZ);
  double partialDerivativeToY = (3.0 * Math::sqr(p.fY) * p.fZ) + (Math::sqr(p.fX) * p.fX);
  double partialDerivativeToZ = (3.0 * Math::sqr(p.fZ) * p.fX) + (Math::sqr(p.fY) * p.fY);

  return Vector3D(partialDerivativeToX,partialDerivativeToY,partialDerivativeToZ);
}
