// --------------------------------------------------
// Filename      : generate-falling-spheres-frame.cpp
// Author        : Sven Maerivoet
// Last modified : 30/08/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// --------------------------------------------------

#include <iostream>
#include <math.h>
#include <string>
#include "opengl/math.h"

// string-streams are not always compatible
// => Microsoft Visual C++ 6.0 and UNIX/Linux are treated separately
#ifdef _WIN32
  #include <sstream>
#else
  #include <strstream>
  #define  stringstream strstream
#endif

using namespace std;

double getValue(string str)
{
  stringstream tmp;
  double number;

  tmp << str;
  if (!(tmp >> number)) {
    cerr << "getValue() -> error getting value [" << str << "]." << endl;
    exit(1);
  }
  else {
    char t;
    if (tmp >> t) {
      cerr << "getValue() -> bum chars in number [" << str << "]." << endl;
      exit(1);
    } 
  }

  return number;
}

void generateFallingSpheresFrame(int part,
                                 int framesPerPart,
                                 int currentFrame)
{
  double minX, maxX, x,
         minY, maxY, y,
         minZ, maxZ, z;
  double fraction = ((double) (currentFrame - 1)) / ((double) (framesPerPart - 1));

  switch (part) {
    case 1 :
      cout << "def Floor {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use          ColorWhite" << endl;

      minX = 0.5;
      maxX = 0.02;
      x = Math::lerp(minX,maxX,fraction);

      cout << "    solidTexture 5 2 0 " << x << " " << x << endl;
      cout << "    scale        5.0 0.001 5.0" << endl;
      cout << "    cube" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 2 :
    case 3 :
    case 4 :
    case 5 :
    case 6 :
    case 7 :
      cout << "def Floor {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use          ColorWhite" << endl;
      cout << "    solidTexture 5 2 0 0.02 0.02" << endl;
      cout << "    scale        5.0 0.001 5.0" << endl;
      cout << "    cube" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;
  }

  cout << endl;

  switch (part) {
    case 3 :
      cout << "def BrickBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use           ColorWhite" << endl;
      cout << "    imageTexture  data/textures/bricks/bricks3.bmp 1 8 2" << endl;

      minY = 3.0;
      maxY = 1.1;
      y = Math::lerp(minY,maxY,fraction);

      cout << "    translate    -1.5 " << y << " 1.0" << endl;
      cout << "    scale         1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 4 :
    case 5 :
    case 6 :
      cout << "def BrickBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use           ColorWhite" << endl;
      cout << "    imageTexture  data/textures/bricks/bricks3.bmp 1 8 2" << endl;
      cout << "    translate    -1.5 1.1 1.0" << endl;
      cout << "    scale         1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 7 :
      cout << "def BrickBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use           ColorWhite" << endl;
      cout << "    imageTexture  data/textures/bricks/bricks3.bmp 1 8 2" << endl;

      minX = -1.5;
      maxX =  6.0;
      x = Math::lerp(minX,maxX,fraction);

      minZ = 1.0;
      maxZ = -6.5;
      z = Math::lerp(minZ,maxZ,fraction);

      cout << "    translate    " << x << " 1.1 " << z << endl;
      cout << "    scale         1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    default :
      break;
  }

  cout << endl;

  switch (part) {
    case 4 :
      cout << "def WaterBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use          ColorWhite" << endl;
      cout << "    imageTexture data/textures/water/water1.bmp 1 2 2" << endl;

      minX = 6.0;
      maxX = 1.5;
      x = Math::lerp(minX,maxX,fraction);

      minY = 6.0;
      maxY = 1.1;
      y = Math::lerp(minY,maxY,fraction);

      cout << "    translate    " << x << " " << y << " 1.0" << endl;
      cout << "    scale        1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 5 :
    case 6 :
      cout << "def WaterBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use          ColorWhite" << endl;
      cout << "    imageTexture data/textures/water/water1.bmp 1 2 2" << endl;
      cout << "    translate    1.5 1.1 1.0" << endl;
      cout << "    scale        1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 7 :
      cout << "def WaterBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use          ColorWhite" << endl;
      cout << "    imageTexture data/textures/water/water1.bmp 1 2 2" << endl;

      minZ = 1.0;
      maxZ = 6.5;
      z = Math::lerp(minZ,maxZ,fraction);

      cout << "    translate    1.5 1.1 " << z << endl;
      cout << "    scale        1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    default :
      break;
  }

  cout << endl;

  switch (part) {
    case 5 :
      cout << "def GlassBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use                ColorRed" << endl;
      cout << "    transparency       0.9" << endl;
      cout << "    retainDiffuseColor 1" << endl;
      cout << "    speedOfLight       1.0" << endl;

      minY = 4.0;
      maxY = 1.1;
      y = Math::lerp(minY,maxY,fraction);

      cout << "    translate          1.5 " << y << " 3.0" << endl;
      cout << "    scale              1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 6 :
      cout << "def GlassBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use                ColorRed" << endl;
      cout << "    transparency       0.9" << endl;
      cout << "    retainDiffuseColor 1" << endl;
      cout << "    speedOfLight       1.0" << endl;

      minX = 1.5;
      maxX = 3.5;
      x = Math::lerp(minX,maxX,fraction);

      minZ = 3.0;
      maxZ = 1.0;
      z = Math::lerp(minZ,maxZ,fraction);

      cout << "    translate          " << x << " 1.1 " << z << endl;
      cout << "    scale              1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    case 7 :
      cout << "def GlassBall {" << endl;
      cout << "  push" << endl;
      cout << "    defaultMaterials" << endl;
      cout << "    use                ColorRed" << endl;
      cout << "    transparency       0.9" << endl;
      cout << "    retainDiffuseColor 1" << endl;
      cout << "    speedOfLight       1.0" << endl;

      minZ = 1.0;
      maxZ = 13.0;
      z = Math::lerp(minZ,maxZ,fraction);

      cout << "    translate          3.5 1.1 " << z << endl;
      cout << "    scale              1.0 1.0 1.0" << endl;
      cout << "    sphere" << endl;
      cout << "  pop" << endl;
      cout << "}" << endl;
      break;

    default :
      break;
  }

  cout << endl;
  cout << "! ===============================" << endl;
  cout << endl;

  cout << "include data/sdl/include/colors.sdl" << endl;

  cout << endl;

  switch (part) {
    case 1 :
      cout << "eyePoint  0.0 5.0 5.0" << endl;
      cout << "viewPoint 0.0 0.0 0.0" << endl;
      break;

    case 2 :
      minX = 0.0;
      maxX = 5.0;
      x = Math::lerp(minX,maxX,fraction);

      cout << "eyePoint  " << x << " 5.0 5.0" << endl;

      minY = 0.0;
      maxY = -2.0;
      y = Math::lerp(minY,maxY,fraction);

      cout << "viewPoint 0.0 " << y << " 0.0" << endl;
      break;

    case 3 :
    case 4 :
    case 5 :
    case 6 :
    case 7 :
      cout << "eyePoint  5.0  5.0 5.0" << endl;
      cout << "viewPoint 0.0 -2.0 0.0" << endl;
      break;
  }

  cout << endl;

  cout << "light -1.0 4.0  4.0 0.5 0.5 0.5" << endl;
  cout << "light  5.0 4.0 -1.0 0.5 0.5 0.5" << endl;
  cout << "light  3.0 1.0  5.0 0.5 0.5 0.5" << endl;
  cout << "light  0.0 9.0  0.0 0.3 0.3 0.3" << endl;

  cout << endl;

  cout << "push use Floor pop" << endl;
  switch (part) {
    case 3 :
      cout << "push use BrickBall pop" << endl;
      break;

    case 4 :
      cout << "push use BrickBall pop" << endl;
      cout << "push use WaterBall pop" << endl;
      break;

    case 5 :
      cout << "push use BrickBall pop" << endl;
      cout << "push use WaterBall pop" << endl;
      cout << "push use GlassBall pop" << endl;
      break;

    case 6 :
      cout << "push use BrickBall pop" << endl;
      cout << "push use WaterBall pop" << endl;
      cout << "push use GlassBall pop" << endl;
      break;

    case 7 :
      cout << "push use BrickBall pop" << endl;
      cout << "push use WaterBall pop" << endl;
      cout << "push use GlassBall pop" << endl;
      break;

    default :
      break;
  }
}

void main(int argc, char* argv[])
{
  if (argc != (1 + 3)) {
    cout << endl;
    cout << "Usage : generate-falling-spheres-frame <part> <frames-per-part> <current-frame>" << endl;
    cout << endl;
    cout << "Part 1 changes the floor's texture, part 2 changes the eye- and viewpoint," << endl;
    cout << "part 3 drops the brickball, part 4 drops the waterball," << endl;
    cout << "part 5 shows the glassball, part 6 moves the glassball and" << endl;
    cout << "part 7 moves all the balls." << endl;
    cout << endl;
    exit(0);
  }

  generateFallingSpheresFrame((int) getValue(argv[1]),(int) getValue(argv[2]),(int) getValue(argv[3]));
}
