// -------------------------------------------------
// Filename      : generate-rotating-earth-frame.cpp
// Author        : Sven Maerivoet
// Last modified : 30/08/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// -------------------------------------------------

#include "opengl/math.h"
#include <iostream>
#include <math.h>
#include <string>

// string-streams are not always compatible
// => Microsoft Visual C++ 6.0 and UNIX/Linux are treated separately
#ifdef _WIN32
  #include <sstream>
#else
  #include <strstream>
  #define  stringstream strstream
#endif

using namespace std;

double getValue(string str)
{
  stringstream tmp;
  double number;

  tmp << str;
  if (!(tmp >> number)) {
    cerr << "getValue() -> error getting value [" << str << "]." << endl;
    exit(1);
  }
  else {
    char t;
    if (tmp >> t) {
      cerr << "getValue() -> bum chars in number [" << str << "]." << endl;
      exit(1);
    } 
  }

  return number;
}

void generateRotatingEarthFrame(int rotationAngle)
{
  cout << "def Stars {" << endl;
  cout << "  push" << endl;
  cout << "    defaultMaterials" << endl;
  cout << "    imageTexture  data/textures/stars/stars5.bmp 0 10 10" << endl;
  cout << "    translate     0.0  0.0 -5.0" << endl;
  cout << "    scale        10.0 10.0  0.1" << endl;
  cout << "    cube" << endl;
  cout << "  pop" << endl;
  cout << "}" << endl;
  cout << endl;
  cout << "def Earth {" << endl;
  cout << "  push" << endl;
  cout << "    defaultMaterials" << endl;
  cout << "    use           ColorWhite" << endl;
  cout << "    imageTexture  data/textures/other/earth.bmp 1 1 1" << endl;
  cout << "    rotate        " << rotationAngle << " 0.0 1.0 0.0" << endl;
  cout << "    rotate       -90.0 1.0 0.0 0.0" << endl;
  cout << "    sphere" << endl;
  cout << "  pop" << endl;
  cout << "}" << endl;
  cout << endl;
  cout << "! ===============================" << endl;
  cout << endl;
  cout << "include data/sdl/include/colors.sdl" << endl;
  cout << endl;
  cout << "eyePoint  0.0 1.0 4.0" << endl;
  cout << "viewPoint 0.0 0.0 0.0" << endl;
  cout << endl;

  double lightRadius = 5.0;

  double yPosition = lightRadius * cos(Math::degToRad(rotationAngle));
  double zPosition = lightRadius * sin(Math::degToRad(rotationAngle));
  cout << "light 0.0 " << yPosition << " " << zPosition << " 1.0 1.0 0.0" << endl;

  yPosition = lightRadius * cos(Math::degToRad((90 + rotationAngle) % 360));
  zPosition = lightRadius * sin(Math::degToRad((90 + rotationAngle) % 360));
  cout << "light 0.0 " << yPosition << " " << zPosition << " 0.0 0.0 1.0" << endl;

  yPosition = lightRadius * cos(Math::degToRad((180 + rotationAngle) % 360));
  zPosition = lightRadius * sin(Math::degToRad((180 + rotationAngle) % 360));
  cout << "light 0.0 " << yPosition << " " << zPosition << " 1.0 0.0 0.0" << endl;

  yPosition = lightRadius * cos(Math::degToRad((270 + rotationAngle) % 360));
  zPosition = lightRadius * sin(Math::degToRad((270 + rotationAngle) % 360));
  cout << "light 0.0 " << yPosition << " " << zPosition << " 1.0 1.0 1.0" << endl;

  cout << endl;
  cout << "push use Stars pop" << endl;
  cout << "push use Earth pop" << endl;
}

void main(int argc, char* argv[])
{
  if (argc != (1 + 1)) {
    cout << endl;
    cout << "Usage : generate-rotating-earth-frame <rotation-angle>" << endl;
    cout << endl;
    exit(0);
  }

  generateRotatingEarthFrame((int) getValue(argv[1]));
}
