// ---------------------------------------------------
// Filename      : generate-rotating-torloid-frame.cpp
// Author        : Sven Maerivoet
// Last modified : 14/06/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// ---------------------------------------------------

#include <iostream>
#include <math.h>
#include <string>

// string-streams are not always compatible
// => Microsoft Visual C++ 6.0 and UNIX/Linux are treated separately
#ifdef _WIN32
  #include <sstream>
#else
  #include <strstream>
  #define  stringstream strstream
#endif

using namespace std;

int kLeftMargin;

double getValue(string str)
{
  stringstream tmp;
  double number;

  tmp << str;
  if (!(tmp >> number)) {
    cerr << "getValue() -> error getting value [" << str << "]." << endl;
    exit(1);
  }
  else {
    char t;
    if (tmp >> t) {
      cerr << "getValue() -> bum chars in number [" << str << "]." << endl;
      exit(1);
    } 
  }

  return number;
}

void makeLeftMargin(void)
{
  for (int i = 0; i < kLeftMargin; ++i) {
    cout << " ";
  }
}

void generateRotatingTorloidFrame(int rotationAngle,
                                  double tubeRadius,
                                  double torusRadius,
                                  double tubeInhibitionFactor,
                                  double torusInhibitionFactor,
                                  int nrOfNewToriAtEachGeneration)
{
  cout << "def Torloid {" << endl;
  cout << "  push" << endl;

  double tubeRadii[4], torusRadii[4];
  int i, j, k;

  tubeRadii[0] = tubeRadius;
  torusRadii[0] = torusRadius;

  for (i = 1; i < 4; ++i) {
    tubeRadii[i] = tubeRadii[i - 1] * tubeInhibitionFactor;
    torusRadii[i] = torusRadii[i - 1] * torusInhibitionFactor;
  }

  // generation 0
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;

  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble8.bmp 1 10 5" << endl;
  makeLeftMargin(); cout << "  rotate           " << rotationAngle << " 1.0 0.0 0.0" << endl;
  makeLeftMargin(); cout << "  torus            " << tubeRadii[0] << " " << torusRadii[0] << endl;
  makeLeftMargin(); cout << "pop" << endl;

  // generation 1
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;
  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble4.bmp 1 10 5" << endl;

  for (i = 0; i < nrOfNewToriAtEachGeneration; ++i) {
    double angle = i * (360.0 / nrOfNewToriAtEachGeneration);
    angle = (((int) angle + rotationAngle) % 360);

    makeLeftMargin(); cout << "  push" << endl;
    makeLeftMargin(); cout << "    rotate    " << rotationAngle << " 1.0 0.0 0.0" << endl;
    makeLeftMargin(); cout << "    rotate    " << angle << " 0.0 0.0 1.0" <<endl;
    makeLeftMargin(); cout << "    translate " << torusRadii[0] << " 0.0 0.0" << endl;
    makeLeftMargin(); cout << "    rotate    -90.0 1.0 0.0 0.0" << endl;
    makeLeftMargin(); cout << "    torus     " << tubeRadii[1] << " " << torusRadii[1] << endl;
    makeLeftMargin(); cout << "  pop" << endl;
  }
  makeLeftMargin(); cout << "pop" << endl;

  // generation 2
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;
  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble8.bmp 1 10 5" << endl;
  for (i = 0; i < nrOfNewToriAtEachGeneration; ++i) {
    for (j = 0; j < nrOfNewToriAtEachGeneration; ++j) {

      double angleGen1 = i * (360.0 / nrOfNewToriAtEachGeneration);
      double angleGen2 = j * (360.0 / nrOfNewToriAtEachGeneration);
      angleGen1 = (((int) angleGen1 + rotationAngle) % 360);
      angleGen2 = (((int) angleGen2 + rotationAngle) % 360);

      makeLeftMargin(); cout << "  push" <<endl;
      makeLeftMargin(); cout << "    rotate    " << rotationAngle << " 1.0 0.0 0.0" << endl;
      makeLeftMargin(); cout << "    rotate    " << angleGen1 << " 0.0 0.0 1.0" <<endl;
      makeLeftMargin(); cout << "    translate " << torusRadii[0] << " 0.0 0.0" << endl;

      makeLeftMargin(); cout << "    rotate    " << angleGen2 << " 0.0 1.0 0.0" <<endl;
      makeLeftMargin(); cout << "    translate " << torusRadii[1] << " 0.0 0.0" << endl;
      makeLeftMargin(); cout << "    torus     " << tubeRadii[2] << " " << torusRadii[2] << endl;
      makeLeftMargin(); cout << "  pop" << endl;
    }
  }
  makeLeftMargin(); cout << "pop" << endl;

  // generation 3
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;
  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble4.bmp 1 10 5" << endl;
  for (i = 0; i < nrOfNewToriAtEachGeneration; ++i) {
    for (j = 0; j < nrOfNewToriAtEachGeneration; ++j) {
      for (k = 0; k < nrOfNewToriAtEachGeneration; ++k) {

        double angleGen1 = i * (360.0 / nrOfNewToriAtEachGeneration);
        double angleGen2 = j * (360.0 / nrOfNewToriAtEachGeneration);
        double angleGen3 = k * (360.0 / nrOfNewToriAtEachGeneration);
        angleGen1 = (((int) angleGen1 + rotationAngle) % 360);
        angleGen2 = (((int) angleGen2 + rotationAngle) % 360);
        angleGen3 = (((int) angleGen3 + rotationAngle) % 360);

        makeLeftMargin(); cout << "  push" <<endl;
        makeLeftMargin(); cout << "    rotate    " << rotationAngle << " 1.0 0.0 0.0" << endl;
        makeLeftMargin(); cout << "    rotate    " << angleGen1 << " 0.0 0.0 1.0" <<endl;
        makeLeftMargin(); cout << "    translate " << torusRadii[0] << " 0.0 0.0" << endl;

        makeLeftMargin(); cout << "    rotate    " << angleGen2 << " 0.0 1.0 0.0" <<endl;
        makeLeftMargin(); cout << "    translate " << torusRadii[1] << " 0.0 0.0" << endl;

        makeLeftMargin(); cout << "    rotate    " << angleGen3 << " 0.0 0.0 1.0" <<endl;
        makeLeftMargin(); cout << "    translate " << torusRadii[2] << " 0.0 0.0" << endl;
        makeLeftMargin(); cout << "    rotate    -90.0 1.0 0.0 0.0" << endl;
        makeLeftMargin(); cout << "    torus     " << tubeRadii[3] << " " << torusRadii[3] << endl;
        makeLeftMargin(); cout << "  pop" << endl;
      }
    }
  }
  makeLeftMargin(); cout << "pop" << endl;

  cout << "  pop" << endl;
  cout << "}" << endl;
  cout << endl;
  cout << "! ===============================" << endl;
  cout << endl;
  cout << "include data/sdl/include/colors.sdl" << endl;
  cout << endl;
  cout << "eyePoint    25.0 15.0 25.0" << endl;
  cout << "viewPoint    0.0  0.0  0.0" << endl;
  cout << "upDirection  0.0  1.0  2.0" << endl;
  cout << endl;
  cout << "showExtents 1" << endl;
  cout << endl;
  cout << "light 40.0 40.0 40.0 1.0 1.0 1.0" << endl;
  cout << endl;
  cout << "push use Torloid pop" << endl;
}

void main(int argc, char* argv[])
{
  if (argc != (1 + 1)) {
    cout << endl;
    cout << "Usage : generate-rotating-torloid-frame <rotation-angle>" << endl;
    cout << endl;
    exit(0);
  }

  kLeftMargin = 4;

  generateRotatingTorloidFrame((int) getValue(argv[1]),2.0,12.0,0.5,0.4,8);
}
