// ------------------------------------
// Filename      : generate-torloid.cpp
// Author        : Sven Maerivoet
// Last modified : 19/05/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// ------------------------------------

#include "opengl/primitives.h"
#include "opengl/rng.h"
#include <iostream>
#include <math.h>
#include <string>

// string-streams are not always compatible
// => Microsoft Visual C++ 6.0 and UNIX/Linux are treated separately
#ifdef _WIN32
  #include <sstream>
#else
  #include <strstream>
  #define  stringstream strstream
#endif

using namespace std;

int kLeftMargin;

double getValue(string str)
{
  stringstream tmp;
  double number;

  tmp << str;
  if (!(tmp >> number)) {
    cerr << "getValue() -> error getting value [" << str << "]." << endl;
    exit(1);
  }
  else {
    char t;
    if (tmp >> t) {
      cerr << "getValue() -> bum chars in number [" << str << "]." << endl;
      exit(1);
    } 
  }

  return number;
}

void makeLeftMargin(void)
{
  for (int i = 0; i < kLeftMargin; ++i) {
    cout << " ";
  }
}

void generateTorloid(double tubeRadius,
                     double torusRadius,
                     double tubeInhibitionFactor,
                     double torusInhibitionFactor,
                     int nrOfNewToriAtEachGeneration)
{
  double tubeRadii[4], torusRadii[4];
  int i, j, k;

  tubeRadii[0] = tubeRadius;
  torusRadii[0] = torusRadius;

  for (i = 1; i < 4; ++i) {
    tubeRadii[i] = tubeRadii[i - 1] * tubeInhibitionFactor;
    torusRadii[i] = torusRadii[i - 1] * torusInhibitionFactor;
  }

  // generation 0
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;

  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble8.bmp 1 10 5" << endl;
  makeLeftMargin(); cout << "  torus            " << tubeRadii[0] << " " << torusRadii[0] << endl;
  makeLeftMargin(); cout << "pop" << endl;

  // generation 1
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;
  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble4.bmp 1 10 5" << endl;

  for (i = 0; i < nrOfNewToriAtEachGeneration; ++i) {
    double angle = i * (360.0 / nrOfNewToriAtEachGeneration);

    makeLeftMargin(); cout << "  push" << endl;
    makeLeftMargin(); cout << "    rotate    " << angle << " 0.0 0.0 1.0" <<endl;
    makeLeftMargin(); cout << "    translate " << torusRadii[0] << " 0.0 0.0" << endl;
    makeLeftMargin(); cout << "    rotate    -90.0 1.0 0.0 0.0" << endl;
    makeLeftMargin(); cout << "    torus     " << tubeRadii[1] << " " << torusRadii[1] << endl;
    makeLeftMargin(); cout << "  pop" << endl;
  }
  makeLeftMargin(); cout << "pop" << endl;

  // generation 2
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;
  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble8.bmp 1 10 5" << endl;
  for (i = 0; i < nrOfNewToriAtEachGeneration; ++i) {
    for (j = 0; j < nrOfNewToriAtEachGeneration; ++j) {

      double angleGen1 = i * (360.0 / nrOfNewToriAtEachGeneration);
      double angleGen2 = j * (360.0 / nrOfNewToriAtEachGeneration);

      makeLeftMargin(); cout << "  push" <<endl;
      makeLeftMargin(); cout << "    rotate    " << angleGen1 << " 0.0 0.0 1.0" <<endl;
      makeLeftMargin(); cout << "    translate " << torusRadii[0] << " 0.0 0.0" << endl;

      makeLeftMargin(); cout << "    rotate    " << angleGen2 << " 0.0 1.0 0.0" <<endl;
      makeLeftMargin(); cout << "    translate " << torusRadii[1] << " 0.0 0.0" << endl;
      makeLeftMargin(); cout << "    torus     " << tubeRadii[2] << " " << torusRadii[2] << endl;
      makeLeftMargin(); cout << "  pop" << endl;
    }
  }
  makeLeftMargin(); cout << "pop" << endl;

  // generation 3
  makeLeftMargin(); cout << "push" << endl;
  makeLeftMargin(); cout << "  defaultMaterials" << endl;
  makeLeftMargin(); cout << "  diffuse          1.0 1.0 1.0" << endl;
  makeLeftMargin(); cout << "  specular         0.7 0.7 0.7" << endl;
  makeLeftMargin(); cout << "  specularExponent 100" << endl;
  makeLeftMargin(); cout << "  imageTexture     data/textures/marble/marble4.bmp 1 10 5" << endl;
  for (i = 0; i < nrOfNewToriAtEachGeneration; ++i) {
    for (j = 0; j < nrOfNewToriAtEachGeneration; ++j) {
      for (k = 0; k < nrOfNewToriAtEachGeneration; ++k) {

        double angleGen1 = i * (360.0 / nrOfNewToriAtEachGeneration);
        double angleGen2 = j * (360.0 / nrOfNewToriAtEachGeneration);
        double angleGen3 = k * (360.0 / nrOfNewToriAtEachGeneration);

        makeLeftMargin(); cout << "  push" <<endl;
        makeLeftMargin(); cout << "    rotate    " << angleGen1 << " 0.0 0.0 1.0" <<endl;
        makeLeftMargin(); cout << "    translate " << torusRadii[0] << " 0.0 0.0" << endl;

        makeLeftMargin(); cout << "    rotate    " << angleGen2 << " 0.0 1.0 0.0" <<endl;
        makeLeftMargin(); cout << "    translate " << torusRadii[1] << " 0.0 0.0" << endl;

        makeLeftMargin(); cout << "    rotate    " << angleGen3 << " 0.0 0.0 1.0" <<endl;
        makeLeftMargin(); cout << "    translate " << torusRadii[2] << " 0.0 0.0" << endl;
        makeLeftMargin(); cout << "    rotate    -90.0 1.0 0.0 0.0" << endl;
        makeLeftMargin(); cout << "    torus     " << tubeRadii[3] << " " << torusRadii[3] << endl;
        makeLeftMargin(); cout << "  pop" << endl;
      }
    }
  }
  makeLeftMargin(); cout << "pop" << endl;
}

void main(int argc, char* argv[])
{
  if (argc != (6 + 1)) {
    cout << endl;
    cout << "Usage : generate-torloid <left-margin> <tube-radius> <torus-radius>" << endl;
    cout << "                         <tube-inhibition-factor> <torus-inhibition-factor>" << endl;
    cout << "                         <nr-of-new-tori-at-each-generation>" << endl;
    cout << endl;
    exit(0);
  }

  kLeftMargin = (int) getValue(argv[1]);

  generateTorloid(getValue(argv[2]),
                  getValue(argv[3]),
                  getValue(argv[4]),
                  getValue(argv[5]),
                  (int) getValue(argv[6]));
}
