// ------------------------------
// Filename      : jitter.cpp
// Author        : Sven Maerivoet
// Last modified : 27/04/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// ------------------------------

#include "jitter.h"

using namespace std;

// ------------------
// class JitterVector
// ------------------

// ------------
// constructors
// ------------
JitterVector::JitterVector(void) : x(0.0),
                                   y(0.0)
{
}

JitterVector::JitterVector(double jvX, double jvY) : x(jvX),
                                                     y(jvY)
{
}

// ----------
// destructor
// ----------
JitterVector::~JitterVector(void)
{
}

// -------------------
// class JitterVectors
// -------------------

// -----------
// constructor
// -----------
JitterVectors::JitterVectors(int nrOfJitterVectors)
{
  // the sum of each jitter-cluster must be zero
  switch (nrOfJitterVectors) {

    case 2 :
      fJitterVectors.resize(2);
      fJitterVectors[0]  = JitterVector( 0.246490, 0.249999);
      fJitterVectors[1]  = JitterVector(-0.246490,-0.249999);
      break;

    case 3 :
      fJitterVectors.resize(3);
      fJitterVectors[0]  = JitterVector(-0.373411,-0.250550);
      fJitterVectors[1]  = JitterVector( 0.256263, 0.368119);
      fJitterVectors[2]  = JitterVector( 0.117148,-0.117570);
      break;

    case 4 :
      fJitterVectors.resize(4);
      fJitterVectors[0]  = JitterVector(-0.208147, 0.353730);
      fJitterVectors[1]  = JitterVector( 0.203849,-0.353780);
      fJitterVectors[2]  = JitterVector(-0.292626,-0.149945);
      fJitterVectors[3]  = JitterVector( 0.296924, 0.149994);
      break;

    case 8 :
      fJitterVectors.resize(8);
      fJitterVectors[0]  = JitterVector(-0.334818, 0.435331);
      fJitterVectors[1]  = JitterVector( 0.286438,-0.393495);
      fJitterVectors[2]  = JitterVector( 0.459462, 0.141540);
      fJitterVectors[3]  = JitterVector(-0.414498,-0.192829);
      fJitterVectors[4]  = JitterVector(-0.183790, 0.082102);
      fJitterVectors[5]  = JitterVector(-0.079263,-0.317383);
      fJitterVectors[6]  = JitterVector( 0.102254, 0.299133);
      fJitterVectors[7]  = JitterVector( 0.164216,-0.054399);
      break;
    
    case 15 :
      fJitterVectors.resize(15);
      fJitterVectors[0]  = JitterVector( 0.285561, 0.188437);
      fJitterVectors[1]  = JitterVector( 0.360176,-0.065688);
      fJitterVectors[2]  = JitterVector(-0.111751, 0.275019);
      fJitterVectors[3]  = JitterVector(-0.055918,-0.215197);
      fJitterVectors[4]  = JitterVector(-0.080231,-0.470965);
      fJitterVectors[5]  = JitterVector( 0.138721, 0.409168);
      fJitterVectors[6]  = JitterVector( 0.384120, 0.458500);
      fJitterVectors[7]  = JitterVector(-0.454968, 0.134088);
      fJitterVectors[8]  = JitterVector( 0.179271,-0.331196);
      fJitterVectors[9]  = JitterVector(-0.307049,-0.364927);
      fJitterVectors[10] = JitterVector( 0.105354,-0.010099);
      fJitterVectors[11] = JitterVector(-0.154180, 0.021794);
      fJitterVectors[12] = JitterVector(-0.370135,-0.116425);
      fJitterVectors[13] = JitterVector( 0.451636,-0.300013);
      fJitterVectors[14] = JitterVector(-0.370610, 0.387504);
      break;

    case 24 :
      fJitterVectors.resize(24);
      fJitterVectors[0]  = JitterVector( 0.030245, 0.136384);
      fJitterVectors[1]  = JitterVector( 0.018865,-0.348867);
      fJitterVectors[2]  = JitterVector(-0.350114,-0.472309);
      fJitterVectors[3]  = JitterVector( 0.222181, 0.149524);
      fJitterVectors[4]  = JitterVector(-0.393670,-0.266873);
      fJitterVectors[5]  = JitterVector( 0.404568, 0.230436);
      fJitterVectors[6]  = JitterVector( 0.098381, 0.465337);
      fJitterVectors[7]  = JitterVector( 0.462671, 0.442116);
      fJitterVectors[8]  = JitterVector( 0.400373,-0.212720);
      fJitterVectors[9]  = JitterVector(-0.409988, 0.263345);
      fJitterVectors[10] = JitterVector(-0.115878,-0.001981);
      fJitterVectors[11] = JitterVector( 0.348425,-0.009237);
      fJitterVectors[12] = JitterVector(-0.464016, 0.066467);
      fJitterVectors[13] = JitterVector(-0.138674,-0.468006);
      fJitterVectors[14] = JitterVector( 0.144932,-0.022780);
      fJitterVectors[15] = JitterVector(-0.250195, 0.150161);
      fJitterVectors[16] = JitterVector(-0.181400,-0.264219);
      fJitterVectors[17] = JitterVector( 0.196097,-0.234139);
      fJitterVectors[18] = JitterVector(-0.311082,-0.078815);
      fJitterVectors[19] = JitterVector( 0.268379, 0.366778);
      fJitterVectors[20] = JitterVector(-0.040601, 0.327109);
      fJitterVectors[21] = JitterVector(-0.234392, 0.354659);
      fJitterVectors[22] = JitterVector(-0.003102,-0.154402);
      fJitterVectors[23] = JitterVector( 0.297997,-0.417965);
              break;

    case 66 :
      fJitterVectors.resize(66);
      fJitterVectors[0]  = JitterVector( 0.266377,-0.218171);
      fJitterVectors[1]  = JitterVector(-0.170919,-0.429368);
      fJitterVectors[2]  = JitterVector( 0.047356,-0.387135);
      fJitterVectors[3]  = JitterVector(-0.430063, 0.363413);
      fJitterVectors[4]  = JitterVector(-0.221638,-0.313768);
      fJitterVectors[5]  = JitterVector( 0.124758,-0.197109);
      fJitterVectors[6]  = JitterVector(-0.400021, 0.482195);
      fJitterVectors[7]  = JitterVector( 0.247882, 0.152010);
      fJitterVectors[8]  = JitterVector(-0.286709,-0.470214);
      fJitterVectors[9]  = JitterVector(-0.426790, 0.004977);
      fJitterVectors[10] = JitterVector(-0.361249,-0.104549);
      fJitterVectors[11] = JitterVector(-0.040643, 0.123453);
      fJitterVectors[12] = JitterVector(-0.189296, 0.438963);
      fJitterVectors[13] = JitterVector(-0.453521,-0.299889);
      fJitterVectors[14] = JitterVector( 0.408216,-0.457699);
      fJitterVectors[15] = JitterVector( 0.328973,-0.101914);
      fJitterVectors[16] = JitterVector(-0.055540,-0.477952);
      fJitterVectors[17] = JitterVector( 0.194421, 0.453510);
      fJitterVectors[18] = JitterVector( 0.404051, 0.224974);
      fJitterVectors[19] = JitterVector( 0.310136, 0.419700);
      fJitterVectors[20] = JitterVector(-0.021743, 0.403898);
      fJitterVectors[21] = JitterVector(-0.466210, 0.248839);
      fJitterVectors[22] = JitterVector( 0.341369, 0.081490);
      fJitterVectors[23] = JitterVector( 0.124156,-0.016859);
      fJitterVectors[24] = JitterVector(-0.461321,-0.176661);
      fJitterVectors[25] = JitterVector( 0.013210, 0.234401);
      fJitterVectors[26] = JitterVector( 0.174258,-0.311854);
      fJitterVectors[27] = JitterVector( 0.294061, 0.263364);
      fJitterVectors[28] = JitterVector(-0.114836, 0.328189);
      fJitterVectors[29] = JitterVector( 0.041206,-0.106205);
      fJitterVectors[30] = JitterVector( 0.079227, 0.345021);
      fJitterVectors[31] = JitterVector(-0.109319,-0.242380);
      fJitterVectors[32] = JitterVector( 0.425005,-0.332397);
      fJitterVectors[33] = JitterVector( 0.009146, 0.015098);
      fJitterVectors[34] = JitterVector(-0.339084,-0.355707);
      fJitterVectors[35] = JitterVector(-0.224596,-0.189548);
      fJitterVectors[36] = JitterVector( 0.083475, 0.117028);
      fJitterVectors[37] = JitterVector( 0.295962,-0.334699);
      fJitterVectors[38] = JitterVector( 0.452998, 0.025397);
      fJitterVectors[39] = JitterVector( 0.206511,-0.104668);
      fJitterVectors[40] = JitterVector( 0.447544,-0.096004);
      fJitterVectors[41] = JitterVector(-0.108006,-0.002471);
      fJitterVectors[42] = JitterVector(-0.380810, 0.130036);
      fJitterVectors[43] = JitterVector(-0.242440, 0.186934);
      fJitterVectors[44] = JitterVector(-0.200363, 0.070863);
      fJitterVectors[45] = JitterVector(-0.344844,-0.230814);
      fJitterVectors[46] = JitterVector( 0.408660, 0.345826);
      fJitterVectors[47] = JitterVector(-0.233016, 0.305203);
      fJitterVectors[48] = JitterVector( 0.158475,-0.430762);
      fJitterVectors[49] = JitterVector( 0.486972, 0.139163);
      fJitterVectors[50] = JitterVector(-0.301610, 0.009319);
      fJitterVectors[51] = JitterVector( 0.282245,-0.458671);
      fJitterVectors[52] = JitterVector( 0.482046, 0.443890);
      fJitterVectors[53] = JitterVector(-0.121527, 0.210223);
      fJitterVectors[54] = JitterVector(-0.477606,-0.424878);
      fJitterVectors[55] = JitterVector(-0.083941,-0.121440);
      fJitterVectors[56] = JitterVector(-0.345773, 0.253779);
      fJitterVectors[57] = JitterVector( 0.234646, 0.034549);
      fJitterVectors[58] = JitterVector( 0.394102,-0.210901);
      fJitterVectors[59] = JitterVector(-0.312571, 0.397656);
      fJitterVectors[60] = JitterVector( 0.200906, 0.333293);
      fJitterVectors[61] = JitterVector( 0.018703,-0.261792);
      fJitterVectors[62] = JitterVector(-0.209349,-0.065383);
      fJitterVectors[63] = JitterVector( 0.076248, 0.478538);
      fJitterVectors[64] = JitterVector(-0.073036,-0.355064);
      fJitterVectors[65] = JitterVector( 0.145087, 0.221726);
      break;
  }
}

// ----------
// destructor
// ----------
JitterVectors::~JitterVectors(void)
{
}

// -------
// methods
// -------
double JitterVectors::getX(int jitterVector) const
{
  return fJitterVectors[jitterVector].x;
}

double JitterVectors::getY(int jitterVector) const
{
  return fJitterVectors[jitterVector].y;
}
