// ------------------------------
// Filename      : raytracer.h
// Author        : Sven Maerivoet
// Last modified : 30/05/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// ------------------------------

#ifndef SM_RAYTRACER_H // include-guard
#define SM_RAYTRACER_H

#include "opengl/chrono.h"
#include "opengl/opengl.h"
#include <string.h>

using namespace std;

class RayTracerGLApp : public TOpenGLApp {
  public:
    // constructor
    RayTracerGLApp(string filename,
                   int blockSize,
                   int nrOfConeSamples,
                   int shadowComputationMode,
                   double shadowFeelerJitterStrength,
                   int antiAliasMode,
                   double jitterStrength,
                   bool normalizeColors,
                   double normalizeBrightness,
                   string outputFilePrefix);

    // destructor
    ~RayTracerGLApp(void);

    // explicit initialization
    void explicitInitialize(void);

    // callback-functions
    void displayFunc(void);

  private:
    // data-members
    string                 fFilename;
    int                    fBlockSize;
    bool                   fComputeShadows;
    EShadowComputationMode fShadowComputationMode;
    double                 fShadowFeelerJitterStrength;
    int                    fNrOfConeSamples;
    bool                   fAntiAlias;
    EAntiAliasMode         fAntiAliasMode;
    bool                   fUseStochasticSampling;
    double                 fJitterStrength;
    bool                   fNormalizeColors;
    double                 fNormalizeBrightness;
    string                 fOutputFilePrefix;
    Chrono                 fChrono;
};

#endif
