// ---------------------------------
// Filename      : sdlviewermain.cpp
// Author        : Sven Maerivoet
// Last modified : 30/05/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// ---------------------------------

#include "sdlviewer.h"
#include "opengl/opengl.h"
#include "opengl/rng.h"
#include <string.h>

// --------------------------------
// global OpenGL Application object
// --------------------------------
TOpenGLApp* fOpenGLApp;

// -------------------------------
// wrappers for callback-functions
// -------------------------------
void displayFuncWrapper(void)                              { fOpenGLApp->antiAliasedDisplayFunc(); }
void reshapeFuncWrapper(int width, int height)             { fOpenGLApp->reshapeFunc(width,height); }
void keyboardFuncWrapper(unsigned char key, int x, int y)  { fOpenGLApp->keyboardFunc(key,x,y); }
void specialFuncWrapper(int key, int x, int y)             { fOpenGLApp->specialFunc(key,x,y); }
void mouseFuncWrapper(int button, int state, int x, int y) { fOpenGLApp->mouseFunc(button,state,x,y); }
void motionFuncWrapper(int x, int y)                       { fOpenGLApp->motionFunc(x,y); }
void passiveMotionFuncWrapper(int x, int y)                { fOpenGLApp->passiveMotionFunc(x,y); }
void idleFuncWrapper(void)                                 { fOpenGLApp->idleFunc(); }

void cleanUp(void)
{
  if (fOpenGLApp != 0) {
    delete fOpenGLApp;
  }
}

void main(int argc, char* argv[])
{
  // register the cleanup-routine
  atexit(&cleanUp);

  string filename = "scene.sdl";
  if (argc > 1) {
    filename = argv[1];
  }

  // create and initialize the OpenGL Application object
  fOpenGLApp = new SDLViewerGLApp(filename);
  fOpenGLApp->initialize(0,0,
                         320,240,
                         "Viewing an SDL-scene",
                         TOpenGLApp::kEnableDoubleBuffering,
                         TOpenGLApp::kEnableDepthBuffer);

  // register the necessary callback-functions
  fOpenGLApp->registerDisplayFunc(displayFuncWrapper);
  fOpenGLApp->registerReshapeFunc(reshapeFuncWrapper);
  fOpenGLApp->registerKeyboardFunc(keyboardFuncWrapper);
  fOpenGLApp->registerSpecialFunc(specialFuncWrapper);
  fOpenGLApp->registerIdleFunc(idleFuncWrapper);

  // activate the event handling
  fOpenGLApp->activateEventHandling();
}
