// ------------------------------
// Filename      : swarm.h
// Author        : Sven Maerivoet
// Last modified : 10/04/2001
// Target        : C++ (OpenGL)
//
// All rights reserved.
// ------------------------------

#ifndef SM_SWARM_H // include-guard
#define SM_SWARM_H

#include "opengl/opengl.h"
#include "opengl/rgbapixmap.h"
#include <vector>

using namespace std;

class Agent {
  public:
    enum EAgentType {kCone,
                     kTeapot,
                     kBorgCube};

    // constructor
    Agent(TOpenGLApp* openGLApp, double size);

    // destructor
    ~Agent(void);

    // data-members
    Point3D     fPosition;
    Vector3D    fVelocity;
    RGBApixmap* fTexture;

    // methods
    void draw(EAgentType agentType);
 
  private:
    // data-members
    TOpenGLApp* fOpenGLApp;
    double      fSize;
};

class Swarm {
  public:
    // constructor
    Swarm(TOpenGLApp* openGLApp,
          const Point3D& centerPosition,
          double positionConstraint,
          double speedConstraint,
          double craziness,
          double agentSize);

    // destructor
    ~Swarm(void);

    // methods
    void moveAgents(void);
    void drawAgents(void);
    void setNrOfAgents(int nrOfAgents);
    int getNrOfAgents(void) const;
    void cycleAgentType(void);

  private:
    // find an agent's nearest neighbour
    Agent findNearestAgent(const Agent& agent) const;

    // data-members
    TOpenGLApp*       fOpenGLApp;
    Point3D           fCenterPosition;
    double            fPositionConstraint;
    double            fSpeedConstraint;
    double            fCraziness;
    double            fAgentSize;
    typedef           vector<Agent> Agents;
    Agents*           fAgents;
    Agent::EAgentType fAgentType;
    RGBApixmap*       fTexture;
};

class SwarmGLApp : public TOpenGLApp {
  public:
    // constructor
    SwarmGLApp(void);

    // destructor
    ~SwarmGLApp(void);

    // explicit initialization
    void explicitInitialize(void);

    // callback-functions
    void displayFunc(void);
    void idleFunc(void);
    void keyboardFunc(unsigned char key, int x, int y);

  private:
    // data-members
    static const double kPositionConstraint;

    bool   fLightMountedOnHead;
    bool   fAdvancing;
    double fToriDelta;
    double fXZTorusSize;
    double fYTorusSize;
    bool   fFogEnabled;
    Swarm* fSwarm;
};

#endif
